package net.sf.sql2java.generator.writer.merge;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.Scanner;

import org.apache.log4j.Logger;

import net.sf.sql2java.common.parameters.StringParser;
import net.sf.sql2java.common.parameters.handlers.ModifierParameterHandler;
import net.sf.sql2java.generator.merger.MergeHelper;
import net.sf.sql2java.generator.writer.CodeWriter;

public class MergeCodeWriter extends CodeWriter {
	private static final Logger LOGGER = Logger.getLogger(MergeCodeWriter.class);
	
	public enum Props { outputPath, conflictPath }
	
	private StringParser parser = new StringParser(new ModifierParameterHandler());
	
// -- Write -------------------------------------------------------------------
	
	@Override
	public void write(String path, String content) throws Exception {
		// -- create the real path
		String realPath = getOutputPath() + File.separator + path;
		String tempPath = getConflictPath() + File.separator + path;
		
		// -- create the result file
		File realFile = new File(realPath);
		
		// -- invoke the modifiers inside the content
		content = parser.parse(content);
		
		// -- check if the file exists
		if (realFile.exists()) 
			handleExistingFile(realFile, new File(tempPath), content);
		else
			writeContent(realFile, content);
	}
	
	protected void handleExistingFile(File originalFile, File generatedFile, String content) throws Exception {
		// -- check if the content is the same
		if (hasContentFast(originalFile, content)) {
			// -- overwrite the existing file
			writeContent(originalFile, content);
		} else {
			// -- write the content to a temporary file
			writeContent(generatedFile, content);
			
			// -- register a new conflict
			addConflict(new Conflict(originalFile, generatedFile));
		}
	}
	
	protected void writeContent(File file, String content) throws Exception {
		// -- check if the file exists
		checkPath(file.getParentFile());
		
		// -- wrap the file into a writer so we can put data in it
        PrintWriter writer = new PrintWriter(new OutputStreamWriter(new FileOutputStream(file)));

        // -- write our content to the file
        writer.write(content);
        writer.flush();
        writer.close();
	}
	
	protected boolean hasContent(File file, String content) throws Exception {
		return hasContent(new Scanner(file), new Scanner(content));
	}
	
	protected boolean hasContent(Scanner fileScanner, Scanner contentScanner) {
		while (fileScanner.hasNextLine()) {
			if (! contentScanner.hasNext()) return false;
			
			String originalLine = fileScanner.nextLine().trim();
			String generatedLine = contentScanner.nextLine().trim();
			
			if (! originalLine.equals(generatedLine))
				return false;
		}
		
		return true;
	}
	
	protected boolean hasContentFast(File file, String content) {
		try {
			// -- get the MD5 hashes for the file and content
			String fileHash = MergeHelper.calculateMD5(file);
			String contentHash = MergeHelper.calculateMD5(content);
			
			return fileHash.equals(contentHash);
		} catch (Exception e) {
			LOGGER.error("Unable to compare files", e);
			return false;
		}
	}
	
	protected void checkPath(File path) {
		if (! path.exists())
			path.mkdirs();
	}
	
// -- Getters -----------------------------------------------------------------
	
	public String getConflictPath() {
		return getProperty(Props.conflictPath.toString());
	}
	
	public String getOutputPath() {
		return getProperty(Props.outputPath.toString());
	}
}
